@@ -27,6 +27,7 @@ from neutron_lib import exceptions
 from neutron_lib import rpc as n_rpc
 from oslo_concurrency import lockutils
 from oslo_config import cfg
+from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_service import loopingcall
@@ -74,6 +75,35 @@ def _wait_if_syncing(f):
     return wrapped
 
 
+class DHCPResourceUpdate(queue.ResourceUpdate):
+
+    def __init__(self, _id, priority, action=None, resource=None,
+                 timestamp=None, tries=5, obj_type=None):
+        super().__init__(_id, priority, action=action, resource=resource,
+                         timestamp=timestamp, tries=tries)
+        self.obj_type = obj_type
+
+    def __lt__(self, other):
+        if other.obj_type == self.obj_type == 'port':
+            # NOTE(ralonsoh): both resources should have "fixed_ips"
+            # information. That key was added to the deleted ports in this
+            # patch but this code runs in the Neutron API (server). Both the
+            # server and the DHCP agent should be updated.
+            # This check could be removed in Y release.
+            if ('fixed_ips' not in self.resource or
+                    'fixed_ips' not in other.resource):
+                return super().__lt__(other)
+
+            self_ips = set(str(fixed_ip['ip_address']) for
+                           fixed_ip in self.resource['fixed_ips'])
+            other_ips = set(str(fixed_ip['ip_address']) for
+                            fixed_ip in other.resource['fixed_ips'])
+            if self_ips & other_ips:
+                return self.timestamp < other.timestamp
+
+        return super().__lt__(other)
+
+
 class DhcpAgent(manager.Manager):
     """DHCP agent service manager.
 
@@ -445,28 +475,28 @@ class DhcpAgent(manager.Manager):
 
     def network_create_end(self, context, payload):
         """Handle the network.create.end notification event."""
-        update = queue.ResourceUpdate(payload['network']['id'],
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_network_create',
-                                      resource=payload)
+        update = DHCPResourceUpdate(payload['network']['id'],
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_network_create',
+                                    resource=payload, obj_type='network')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _network_create(self, payload):
         network_id = payload['network']['id']
         self.enable_dhcp_helper(network_id)
 
     def network_update_end(self, context, payload):
         """Handle the network.update.end notification event."""
-        update = queue.ResourceUpdate(payload['network']['id'],
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_network_update',
-                                      resource=payload)
+        update = DHCPResourceUpdate(payload['network']['id'],
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_network_update',
+                                    resource=payload, obj_type='network')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _network_update(self, payload):
         network_id = payload['network']['id']
         if payload['network']['admin_state_up']:
@@ -476,28 +506,28 @@ class DhcpAgent(manager.Manager):
 
     def network_delete_end(self, context, payload):
         """Handle the network.delete.end notification event."""
-        update = queue.ResourceUpdate(payload['network_id'],
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_network_delete',
-                                      resource=payload)
+        update = DHCPResourceUpdate(payload['network_id'],
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_network_delete',
+                                    resource=payload, obj_type='network')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _network_delete(self, payload):
         network_id = payload['network_id']
         self.disable_dhcp_helper(network_id)
 
     def subnet_update_end(self, context, payload):
         """Handle the subnet.update.end notification event."""
-        update = queue.ResourceUpdate(payload['subnet']['network_id'],
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_subnet_update',
-                                      resource=payload)
+        update = DHCPResourceUpdate(payload['subnet']['network_id'],
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_subnet_update',
+                                    resource=payload, obj_type='subnet')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _subnet_update(self, payload):
         network_id = payload['subnet']['network_id']
         self.refresh_dhcp_helper(network_id)
@@ -528,14 +558,14 @@ class DhcpAgent(manager.Manager):
         network_id = self._get_network_lock_id(payload)
         if not network_id:
             return
-        update = queue.ResourceUpdate(network_id,
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_subnet_delete',
-                                      resource=payload)
+        update = DHCPResourceUpdate(network_id,
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_subnet_delete',
+                                    resource=payload, obj_type='subnet')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _subnet_delete(self, payload):
         network_id = self._get_network_lock_id(payload)
         if not network_id:
@@ -572,17 +602,19 @@ class DhcpAgent(manager.Manager):
     def port_update_end(self, context, payload):
         """Handle the port.update.end notification event."""
         updated_port = dhcp.DictModel(payload['port'])
+        if not dhcp.port_requires_dhcp_configuration(updated_port):
+            return
         if self.cache.is_port_message_stale(updated_port):
             LOG.debug("Discarding stale port update: %s", updated_port)
             return
-        update = queue.ResourceUpdate(updated_port.network_id,
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_port_update',
-                                      resource=updated_port)
+        update = DHCPResourceUpdate(updated_port.network_id,
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_port_update',
+                                    resource=updated_port, obj_type='port')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _port_update(self, updated_port):
         if self.cache.is_port_message_stale(updated_port):
             LOG.debug("Discarding stale port update: %s", updated_port)
@@ -594,7 +626,10 @@ class DhcpAgent(manager.Manager):
         self.reload_allocations(updated_port, network, prio=True)
 
     def reload_allocations(self, port, network, prio=False):
-        LOG.info("Trigger reload_allocations for port %s", port)
+        LOG.info("Trigger reload_allocations for port %s on network %s",
+                 port, network)
+        if not dhcp.port_requires_dhcp_configuration(port):
+            return
         driver_action = 'reload_allocations'
         if self._is_port_on_this_agent(port):
             orig = self.cache.get_port_by_id(port['id'])
@@ -633,14 +668,16 @@ class DhcpAgent(manager.Manager):
     def port_create_end(self, context, payload):
         """Handle the port.create.end notification event."""
         created_port = dhcp.DictModel(payload['port'])
-        update = queue.ResourceUpdate(created_port.network_id,
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_port_create',
-                                      resource=created_port)
+        if not dhcp.port_requires_dhcp_configuration(created_port):
+            return
+        update = DHCPResourceUpdate(created_port.network_id,
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_port_create',
+                                    resource=created_port, obj_type='port')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _port_create(self, created_port):
         network = self.cache.get_network_by_id(created_port.network_id)
         if not network:
@@ -655,9 +692,18 @@ class DhcpAgent(manager.Manager):
             if (new_ips.intersection(cached_ips) and
                 (created_port['id'] != port_cached['id'] or
                  created_port['mac_address'] != port_cached['mac_address'])):
-                self.schedule_resync("Duplicate IP addresses found, "
-                                     "DHCP cache is out of sync",
-                                     created_port.network_id)
+                resync_reason = (
+                    "Duplicate IP addresses found, "
+                    "Port in cache: {cache_port_id}, "
+                    "Created port: {port_id}, "
+                    "IPs in cache: {cached_ips}, "
+                    "new IPs: {new_ips}."
+                    "DHCP cache is out of sync").format(
+                        cache_port_id=port_cached['id'],
+                        port_id=created_port['id'],
+                        cached_ips=cached_ips,
+                        new_ips=new_ips)
+                self.schedule_resync(resync_reason, created_port.network_id)
                 return
         self.reload_allocations(created_port, network, prio=True)
 
@@ -666,14 +712,14 @@ class DhcpAgent(manager.Manager):
         network_id = self._get_network_lock_id(payload)
         if not network_id:
             return
-        update = queue.ResourceUpdate(network_id,
-                                      payload.get('priority',
-                                                  DEFAULT_PRIORITY),
-                                      action='_port_delete',
-                                      resource=payload)
+        update = DHCPResourceUpdate(network_id,
+                                    payload.get('priority', DEFAULT_PRIORITY),
+                                    action='_port_delete',
+                                    resource=payload, obj_type='port')
         self._queue.add(update)
 
     @_wait_if_syncing
+    @log_helpers.log_method_call
     def _port_delete(self, payload):
         network_id = self._get_network_lock_id(payload)
         if not network_id:
