@@ -18,19 +18,17 @@
 
 """Utilities and helper functions."""
 
-import collections
-import decimal
-import errno
+import datetime
 import functools
 import importlib
-import multiprocessing
 import os
 import os.path
 import random
+import re
 import signal
 import socket
 import sys
-import tempfile
+import threading
 import time
 import uuid
 
@@ -38,34 +36,82 @@ import eventlet
 from eventlet.green import subprocess
 import netaddr
 from neutron_lib import constants as n_const
-from oslo_concurrency import lockutils
+from neutron_lib.db import api as db_api
+from neutron_lib import exceptions as n_exc
+from neutron_lib.services.trunk import constants as trunk_constants
+from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
-from oslo_utils import importutils
-import six
-from stevedore import driver
+from oslo_utils import timeutils
+from oslo_utils import uuidutils
+from osprofiler import profiler
+import pkg_resources
 
 import neutron
-from neutron._i18n import _, _LE
-from neutron.db import api as db_api
+from neutron._i18n import _
+from neutron.api import api_common
 
 TIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
 LOG = logging.getLogger(__name__)
-SYNCHRONIZED_PREFIX = 'neutron-'
 
-synchronized = lockutils.synchronized_with_prefix(SYNCHRONIZED_PREFIX)
+DEFAULT_THROTTLER_VALUE = 2
+
+_SEPARATOR_REGEX = re.compile(r'[/\\]+')
 
 
-def ensure_dir(dir_path):
-    """Ensure a directory with 755 permissions mode."""
-    try:
-        os.makedirs(dir_path, 0o755)
-    except OSError as e:
-        # If the directory already existed, don't raise the error.
-        if e.errno != errno.EEXIST:
-            raise
+class WaitTimeout(Exception):
+    """Default exception coming from wait_until_true() function."""
+
+
+class TimerTimeout(n_exc.NeutronException):
+    message = _('Timer timeout expired after %(timeout)s second(s).')
+
+
+class LockWithTimer(object):
+    def __init__(self, threshold):
+        self._threshold = threshold
+        self.timestamp = 0
+        self._lock = threading.Lock()
+
+    def acquire(self):
+        return self._lock.acquire(False)
+
+    def release(self):
+        return self._lock.release()
+
+    def time_to_wait(self):
+        return self.timestamp - time.time() + self._threshold
+
+
+# REVISIT(jlibosva): Some parts of throttler may be similar to what
+#                    neutron.notifiers.batch_notifier.BatchNotifier does. They
+#                    could be refactored and unified.
+def throttler(threshold=DEFAULT_THROTTLER_VALUE):
+    """Throttle number of calls to a function to only once per 'threshold'.
+    """
+    def decorator(f):
+        lock_with_timer = LockWithTimer(threshold)
+
+        @functools.wraps(f)
+        def wrapper(*args, **kwargs):
+            if lock_with_timer.acquire():
+                try:
+                    fname = f.__name__
+                    time_to_wait = lock_with_timer.time_to_wait()
+                    if time_to_wait > 0:
+                        LOG.debug("Call of function %s scheduled, sleeping "
+                                  "%.1f seconds", fname, time_to_wait)
+                        # Decorated function has been called recently, wait.
+                        eventlet.sleep(time_to_wait)
+                    lock_with_timer.timestamp = time.time()
+                finally:
+                    lock_with_timer.release()
+                LOG.debug("Calling throttled function %s", fname)
+                return f(*args, **kwargs)
+        return wrapper
+    return decorator
 
 
 def _subprocess_setup():
@@ -82,125 +128,14 @@ def subprocess_popen(args, stdin=None, stdout=None, stderr=None, shell=False,
                             close_fds=close_fds, env=env)
 
 
-def parse_mappings(mapping_list, unique_values=True, unique_keys=True):
-    """Parse a list of mapping strings into a dictionary.
-
-    :param mapping_list: a list of strings of the form '<key>:<value>'
-    :param unique_values: values must be unique if True
-    :param unique_keys: keys must be unique if True, else implies that keys
-    and values are not unique
-    :returns: a dict mapping keys to values or to list of values
-    """
-    mappings = {}
-    for mapping in mapping_list:
-        mapping = mapping.strip()
-        if not mapping:
-            continue
-        split_result = mapping.split(':')
-        if len(split_result) != 2:
-            raise ValueError(_("Invalid mapping: '%s'") % mapping)
-        key = split_result[0].strip()
-        if not key:
-            raise ValueError(_("Missing key in mapping: '%s'") % mapping)
-        value = split_result[1].strip()
-        if not value:
-            raise ValueError(_("Missing value in mapping: '%s'") % mapping)
-        if unique_keys:
-            if key in mappings:
-                raise ValueError(_("Key %(key)s in mapping: '%(mapping)s' not "
-                                   "unique") % {'key': key,
-                                                'mapping': mapping})
-            if unique_values and value in mappings.values():
-                raise ValueError(_("Value %(value)s in mapping: '%(mapping)s' "
-                                   "not unique") % {'value': value,
-                                                    'mapping': mapping})
-            mappings[key] = value
-        else:
-            mappings.setdefault(key, [])
-            if value not in mappings[key]:
-                mappings[key].append(value)
-    return mappings
-
-
-def get_hostname():
-    return socket.gethostname()
-
-
 def get_first_host_ip(net, ip_version):
     return str(netaddr.IPAddress(net.first + 1, ip_version))
 
 
-def compare_elements(a, b):
-    """Compare elements if a and b have same elements.
-
-    This method doesn't consider ordering
-    """
-    if a is None:
-        a = []
-    if b is None:
-        b = []
-    return set(a) == set(b)
-
-
-def safe_sort_key(value):
-    """Return value hash or build one for dictionaries."""
-    if isinstance(value, collections.Mapping):
-        return sorted(value.items())
-    return value
-
-
-def dict2str(dic):
-    return ','.join("%s=%s" % (key, val)
-                    for key, val in sorted(six.iteritems(dic)))
-
-
-def str2dict(string):
-    res_dict = {}
-    for keyvalue in string.split(','):
-        (key, value) = keyvalue.split('=', 1)
-        res_dict[key] = value
-    return res_dict
-
-
-def dict2tuple(d):
-    items = list(d.items())
-    items.sort()
-    return tuple(items)
-
-
-def diff_list_of_dict(old_list, new_list):
-    new_set = set([dict2str(l) for l in new_list])
-    old_set = set([dict2str(l) for l in old_list])
-    added = new_set - old_set
-    removed = old_set - new_set
-    return [str2dict(a) for a in added], [str2dict(r) for r in removed]
-
-
-def is_extension_supported(plugin, ext_alias):
-    return ext_alias in getattr(
-        plugin, "supported_extension_aliases", [])
-
-
 def log_opt_values(log):
     cfg.CONF.log_opt_values(log, logging.DEBUG)
 
 
-def get_random_mac(base_mac):
-    mac = [int(base_mac[0], 16), int(base_mac[1], 16),
-           int(base_mac[2], 16), random.randint(0x00, 0xff),
-           random.randint(0x00, 0xff), random.randint(0x00, 0xff)]
-    if base_mac[3] != '00':
-        mac[3] = int(base_mac[3], 16)
-    return ':'.join(["%02x" % x for x in mac])
-
-
-def get_random_string(length):
-    """Get a random hex string of the specified length.
-    """
-
-    return "{0:0{1}x}".format(random.getrandbits(length * 4), length)
-
-
 def get_dhcp_agent_device_id(network_id, host):
     # Split host so as to always use only the hostname and
     # not the domain name. This will guarantee consistency
@@ -210,13 +145,6 @@ def get_dhcp_agent_device_id(network_id, host):
     return 'dhcp%s-%s' % (host_uuid, network_id)
 
 
-def cpu_count():
-    try:
-        return multiprocessing.cpu_count()
-    except NotImplementedError:
-        return 1
-
-
 class exception_logger(object):
     """Wrap a function and log raised exception
 
@@ -243,16 +171,19 @@ class exception_logger(object):
         return call
 
 
-def get_other_dvr_serviced_device_owners():
+def get_other_dvr_serviced_device_owners(host_dvr_for_dhcp=True):
     """Return device_owner names for ports that should be serviced by DVR
 
     This doesn't return DEVICE_OWNER_COMPUTE_PREFIX since it is a
     prefix, not a complete device_owner name, so should be handled
     separately (see is_dvr_serviced() below)
     """
-    return [n_const.DEVICE_OWNER_LOADBALANCER,
-            n_const.DEVICE_OWNER_LOADBALANCERV2,
-            n_const.DEVICE_OWNER_DHCP]
+    device_owners = [n_const.DEVICE_OWNER_LOADBALANCER,
+                     n_const.DEVICE_OWNER_LOADBALANCERV2,
+                     trunk_constants.TRUNK_SUBPORT_OWNER]
+    if host_dvr_for_dhcp:
+        device_owners.append(n_const.DEVICE_OWNER_DHCP)
+    return device_owners
 
 
 def get_dvr_allowed_address_pair_device_owners():
@@ -282,6 +213,15 @@ def is_dvr_serviced(device_owner):
             device_owner in get_other_dvr_serviced_device_owners())
 
 
+def is_fip_serviced(device_owner):
+    """Check if the port can be assigned a floating IP
+
+    Helper function to check the device owner of a
+    port can be assigned a floating IP.
+    """
+    return device_owner != n_const.DEVICE_OWNER_DHCP
+
+
 def ip_to_cidr(ip, prefix=None):
     """Convert an ip with no prefix to cidr notation
 
@@ -296,6 +236,23 @@ def ip_to_cidr(ip, prefix=None):
     return str(net)
 
 
+def cidr_to_ip(ip_cidr):
+    """Strip the cidr notation from an ip cidr or ip
+
+    :param ip_cidr: An ipv4 or ipv6 address, with or without cidr notation
+    """
+    net = netaddr.IPNetwork(ip_cidr)
+    return str(net.ip)
+
+
+def cidr_mask(ip_cidr):
+    """Returns the subnet mask length from a cidr
+
+    :param ip_cidr: An ipv4 or ipv6 cidr mask length
+    """
+    return netaddr.IPNetwork(ip_cidr).netmask.netmask_bits()
+
+
 def fixed_ip_cidrs(fixed_ips):
     """Create a list of a port's fixed IPs in cidr notation.
 
@@ -321,6 +278,33 @@ def is_cidr_host(cidr):
     return net.prefixlen == n_const.IPv6_BITS
 
 
+def cidr_mask_length(cidr):
+    """Returns the mask length of a cidr
+
+    :param cidr: (string) either an ipv4 or ipv6 cidr or a host IP.
+    :returns: (int) mask length of a cidr; in case of host IP, the mask length
+              will be 32 (IPv4) or 128 (IPv6)
+    """
+    return netaddr.IPNetwork(cidr).netmask.netmask_bits()
+
+
+def cidr_broadcast_address(cidr):
+    """Returns the broadcast address of a cidr
+
+    :param cidr: (string, netaddr.IPNetwork, netaddr.IPAddress) either an ipv4
+                 or ipv6 cidr or a host IP.
+    :returns: (string) broadcast address of the cidr, None if the cidr has no
+              broadcast domain
+    """
+    broadcast = netaddr.IPNetwork(cidr).broadcast
+    if broadcast:
+        return str(broadcast)
+
+
+def get_ip_version(ip_or_cidr):
+    return netaddr.IPNetwork(ip_or_cidr).version
+
+
 def ip_version_from_int(ip_version_int):
     if ip_version_int == 4:
         return n_const.IPv4
@@ -329,13 +313,22 @@ def ip_version_from_int(ip_version_int):
     raise ValueError(_('Illegal IP version number'))
 
 
-def is_port_trusted(port):
-    """Used to determine if port can be trusted not to attack network.
+def get_network_length(ip_version):
+    """Returns the network length depeding on the IP version"""
+    return (n_const.IPv4_BITS if ip_version == n_const.IP_VERSION_4
+            else n_const.IPv6_BITS)
 
-    Trust is currently based on the device_owner field starting with 'network:'
-    since we restrict who can use that in the default policy.json file.
-    """
-    return port['device_owner'].startswith(n_const.DEVICE_OWNER_NETWORK_PREFIX)
+
+def get_socket_address_family(ip_version):
+    """Returns the address family depending on the IP version"""
+    return (int(socket.AF_INET if ip_version == n_const.IP_VERSION_4
+                else socket.AF_INET6))
+
+
+def is_version_greater_equal(version1, version2):
+    """Returns True if version1 is greater or equal than version2 else False"""
+    return (pkg_resources.parse_version(version1) >=
+            pkg_resources.parse_version(version2))
 
 
 class DelayedStringRenderer(object):
@@ -355,70 +348,6 @@ class DelayedStringRenderer(object):
         return str(self.function(*self.args, **self.kwargs))
 
 
-def camelize(s):
-    return ''.join(s.replace('_', ' ').title().split())
-
-
-def round_val(val):
-    # we rely on decimal module since it behaves consistently across Python
-    # versions (2.x vs. 3.x)
-    return int(decimal.Decimal(val).quantize(decimal.Decimal('1'),
-                                             rounding=decimal.ROUND_HALF_UP))
-
-
-def replace_file(file_name, data, file_mode=0o644):
-    """Replaces the contents of file_name with data in a safe manner.
-
-    First write to a temp file and then rename. Since POSIX renames are
-    atomic, the file is unlikely to be corrupted by competing writes.
-
-    We create the tempfile on the same device to ensure that it can be renamed.
-    """
-
-    base_dir = os.path.dirname(os.path.abspath(file_name))
-    with tempfile.NamedTemporaryFile('w+',
-                                     dir=base_dir,
-                                     delete=False) as tmp_file:
-        tmp_file.write(data)
-    os.chmod(tmp_file.name, file_mode)
-    os.rename(tmp_file.name, file_name)
-
-
-def load_class_by_alias_or_classname(namespace, name):
-    """Load class using stevedore alias or the class name
-    :param namespace: namespace where the alias is defined
-    :param name: alias or class name of the class to be loaded
-    :returns class if calls can be loaded
-    :raises ImportError if class cannot be loaded
-    """
-
-    if not name:
-        LOG.error(_LE("Alias or class name is not set"))
-        raise ImportError(_("Class not found."))
-    try:
-        # Try to resolve class by alias
-        mgr = driver.DriverManager(namespace, name)
-        class_to_load = mgr.driver
-    except RuntimeError:
-        e1_info = sys.exc_info()
-        # Fallback to class name
-        try:
-            class_to_load = importutils.import_class(name)
-        except (ImportError, ValueError):
-            LOG.error(_LE("Error loading class by alias"),
-                      exc_info=e1_info)
-            LOG.error(_LE("Error loading class by class name"),
-                      exc_info=True)
-            raise ImportError(_("Class not found."))
-    return class_to_load
-
-
-def safe_decode_utf8(s):
-    if six.PY3 and isinstance(s, bytes):
-        return s.decode('utf-8', 'surrogateescape')
-    return s
-
-
 def _hex_format(port, mask=0):
 
     def hex_str(num):
@@ -429,7 +358,8 @@ def _hex_format(port, mask=0):
 
 
 def _gen_rules_port_min(port_min, top_bit):
-    """
+    """Generate rules for port_min
+
     Encode a port range range(port_min, (port_min | (top_bit - 1)) + 1) into
     a set of bit value/masks.
     """
@@ -510,7 +440,8 @@ def _gen_rules_port_min(port_min, top_bit):
 
 
 def _gen_rules_port_max(port_max, top_bit):
-    """
+    """Generate rules for port_max
+
     Encode a port range range(port_max & ~(top_bit - 1), port_max + 1) into
     a set of bit value/masks.
     """
@@ -684,7 +615,7 @@ def create_object_with_dependency(creator, dep_getter, dep_creator,
     This function protects against all of the cases where the dependency can
     be concurrently removed by catching exceptions and restarting the
     process of creating the dependency if one no longer exists. It will
-    give up after neutron.db.api.MAX_RETRIES and raise the exception it
+    give up after neutron_lib.db.api.MAX_RETRIES and raise the exception it
     encounters after that.
     """
     result, dependency, dep_id, made_locally = None, None, None, False
@@ -727,7 +658,7 @@ def create_object_with_dependency(creator, dep_getter, dep_creator,
                     try:
                         dep_deleter(dependency)
                     except Exception:
-                        LOG.exception(_LE("Failed cleaning up dependency %s"),
+                        LOG.exception("Failed cleaning up dependency %s",
                                       dep_id)
     return result, dependency
 
@@ -758,55 +689,24 @@ def transaction_guard(f):
 
 
 def wait_until_true(predicate, timeout=60, sleep=1, exception=None):
-    """
-    Wait until callable predicate is evaluated as True
+    """Wait until callable predicate is evaluated as True
 
     :param predicate: Callable deciding whether waiting should continue.
     Best practice is to instantiate predicate with functools.partial()
     :param timeout: Timeout in seconds how long should function wait.
     :param sleep: Polling interval for results in seconds.
-    :param exception: Exception class for eventlet.Timeout.
-    (see doc for eventlet.Timeout for more information)
+    :param exception: Exception instance to raise on timeout. If None is passed
+                      (default) then WaitTimeout exception is raised.
     """
-    with eventlet.timeout.Timeout(timeout, exception):
-        while not predicate():
-            eventlet.sleep(sleep)
-
-
-class _AuthenticBase(object):
-    def __init__(self, addr, **kwargs):
-        super(_AuthenticBase, self).__init__(addr, **kwargs)
-        self._initial_value = addr
-
-    def __str__(self):
-        if isinstance(self._initial_value, six.string_types):
-            return self._initial_value
-        return super(_AuthenticBase, self).__str__()
-
-    # NOTE(ihrachys): override deepcopy because netaddr.* classes are
-    # slot-based and hence would not copy _initial_value
-    def __deepcopy__(self, memo):
-        return self.__class__(self._initial_value)
-
-
-class AuthenticEUI(_AuthenticBase, netaddr.EUI):
-    '''
-    This class retains the format of the MAC address string passed during
-    initialization.
-
-    This is useful when we want to make sure that we retain the format passed
-    by a user through API.
-    '''
-
-
-class AuthenticIPNetwork(_AuthenticBase, netaddr.IPNetwork):
-    '''
-    This class retains the format of the IP network string passed during
-    initialization.
-
-    This is useful when we want to make sure that we retain the format passed
-    by a user through API.
-    '''
+    try:
+        with eventlet.Timeout(timeout):
+            while not predicate():
+                eventlet.sleep(sleep)
+    except eventlet.Timeout:
+        if exception is not None:
+            # pylint: disable=raising-bad-type
+            raise exception
+        raise WaitTimeout(_("Timed out after %d seconds") % timeout)
 
 
 class classproperty(object):
@@ -828,7 +728,7 @@ def attach_exc_details(e, msg, args=_NO_ARGS_MARKER):
 def extract_exc_details(e):
     for attr in ('_error_context_msg', '_error_context_args'):
         if not hasattr(e, attr):
-            return _LE('No details.')
+            return u'No details.'
     details = e._error_context_msg
     args = e._error_context_args
     if args is _NO_ARGS_MARKER:
@@ -838,6 +738,7 @@ def extract_exc_details(e):
 
 def import_modules_recursively(topdir):
     '''Import and return all modules below the topdir directory.'''
+    topdir = _SEPARATOR_REGEX.sub('/', topdir)
     modules = []
     for root, dirs, files in os.walk(topdir):
         for file_ in files:
@@ -848,7 +749,7 @@ def import_modules_recursively(topdir):
             if module == '__init__':
                 continue
 
-            import_base = root.replace('/', '.')
+            import_base = _SEPARATOR_REGEX.sub('.', root)
 
             # NOTE(ihrachys): in Python3, or when we are not located in the
             # directory containing neutron code, __file__ is absolute, so we
@@ -891,13 +792,248 @@ def get_related_rand_names(prefixes, max_length=None):
     if max_length:
         length = max_length - max(len(p) for p in prefixes)
         if length <= 0:
-            raise ValueError("'max_length' must be longer than all prefixes")
+            raise ValueError(
+                _("'max_length' must be longer than all prefixes"))
     else:
         length = 8
-    rndchrs = get_random_string(length)
+    rndchrs = helpers.get_random_string(length)
     return [p + rndchrs for p in prefixes]
 
 
 def get_related_rand_device_names(prefixes):
     return get_related_rand_names(prefixes,
                                   max_length=n_const.DEVICE_NAME_MAX_LEN)
+
+
+def bytes_to_bits(value):
+    return value * 8
+
+
+def bits_to_kilobits(value, base):
+    # NOTE(slaweq): round up that even 1 bit will give 1 kbit as a result
+    return int((value + (base - 1)) / base)
+
+
+def disable_extension_by_service_plugin(core_plugin, service_plugin):
+    if ('filter-validation' in core_plugin.supported_extension_aliases and
+            not api_common.is_filter_validation_supported(service_plugin)):
+        core_plugin.supported_extension_aliases.remove('filter-validation')
+        LOG.info('Disable filter validation extension by service plugin '
+                 '%s.', service_plugin.__class__.__name__)
+
+
+def get_port_fixed_ips_set(port):
+    return set([ip["ip_address"] for ip in port.get("fixed_ips", [])])
+
+
+def port_ip_changed(new_port, original_port):
+    if not new_port or not original_port:
+        return False
+
+    # Quantity is not same, so it is changed.
+    if (len(new_port.get("fixed_ips", [])) !=
+            len(original_port.get("fixed_ips", []))):
+        return True
+
+    # IPs can be placed in any order, so use python set to verify the
+    # fixed IP addresses.
+    if (get_port_fixed_ips_set(new_port) !=
+            get_port_fixed_ips_set(original_port)):
+        return True
+
+    return False
+
+
+def validate_rp_bandwidth(rp_bandwidths, device_names):
+    """Validate resource provider bandwidths against device names.
+
+    :param rp_bandwidths: Dict containing resource provider bandwidths,
+                          in the form:
+                          {'phy1': {'ingress': 100, 'egress': 100}}
+    :param device_names: A set of the device names given in bridge_mappings
+                         in case of ovs-agent or in physical_device_mappings
+                         in case of sriov-agent
+    :raises ValueError: In case of the devices (keys) in the rp_bandwidths dict
+                        are not in the device_names set.
+    """
+
+    for dev_name in rp_bandwidths:
+        if dev_name not in device_names:
+            raise ValueError(_(
+                "Invalid resource_provider_bandwidths: "
+                "Device name %(dev_name)s is missing from "
+                "device mappings") % {'dev_name': dev_name})
+
+
+class Timer(object):
+    """Timer context manager class
+
+    This class creates a context that:
+    - Triggers a timeout exception if the timeout is set.
+    - Returns the time elapsed since the context was initialized.
+    - Returns the time spent in the context once it's closed.
+
+    The timeout exception can be suppressed; when the time expires, the context
+    finishes without rising TimerTimeout.
+
+    NOTE(ralonsoh): this class, when a timeout is defined, cannot be used in
+    other than the main thread. When a timeout is defined, an alarm signal is
+    set. Only the main thread is allowed to set a signal handler and the signal
+    handlers are always executed in this main thread [1].
+    [1] https://docs.python.org/3/library/signal.html#signals-and-threads
+    """
+    def __init__(self, timeout=None, raise_exception=True):
+        self.start = self.delta = None
+        self._timeout = int(timeout) if timeout else None
+        self._timeout_flag = False
+        self._raise_exception = raise_exception
+
+    def _timeout_handler(self, *_):
+        self._timeout_flag = True
+        if self._raise_exception:
+            raise TimerTimeout(timeout=self._timeout)
+        self.__exit__()
+
+    def __enter__(self):
+        self.start = datetime.datetime.now()
+        if self._timeout:
+            signal.signal(signal.SIGALRM, self._timeout_handler)
+            signal.alarm(self._timeout)
+        return self
+
+    def __exit__(self, *_):
+        if self._timeout:
+            signal.alarm(0)
+        self.delta = datetime.datetime.now() - self.start
+
+    def __getattr__(self, item):
+        return getattr(self.delta, item)
+
+    def __iter__(self):
+        self._raise_exception = False
+        return self.__enter__()
+
+    def next(self):  # pragma: no cover
+        # NOTE(ralonsoh): Python 2 support.
+        if not self._timeout_flag:
+            return datetime.datetime.now()
+        raise StopIteration()
+
+    def __next__(self):  # pragma: no cover
+        # NOTE(ralonsoh): Python 3 support.
+        return self.next()
+
+    def __del__(self):
+        signal.alarm(0)
+
+    @property
+    def delta_time_sec(self):
+        return (datetime.datetime.now() - self.start).total_seconds()
+
+
+def collect_profiler_info():
+    p = profiler.get()
+    if p:
+        return {
+            "hmac_key": p.hmac_key,
+            "base_id": p.get_base_id(),
+            "parent_id": p.get_id(),
+        }
+
+
+def spawn(func, *args, **kwargs):
+    """As eventlet.spawn() but with osprofiler initialized in the new threads
+
+    osprofiler stores the profiler instance in thread local storage, therefore
+    in new threads (including eventlet threads) osprofiler comes uninitialized
+    by default. This spawn() is a stand-in replacement for eventlet.spawn()
+    but we re-initialize osprofiler in threads spawn()-ed.
+    """
+
+    profiler_info = collect_profiler_info()
+
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        if profiler_info:
+            profiler.init(**profiler_info)
+        return func(*args, **kwargs)
+
+    return eventlet.spawn(wrapper, *args, **kwargs)
+
+
+def spawn_n(func, *args, **kwargs):
+    """See spawn() above"""
+
+    profiler_info = collect_profiler_info()
+
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        if profiler_info:
+            profiler.init(**profiler_info)
+        return func(*args, **kwargs)
+
+    return eventlet.spawn_n(wrapper, *args, **kwargs)
+
+
+def timecost(f):
+    call_id = uuidutils.generate_uuid()
+    message_base = ("Time-cost: call %(call_id)s function %(fname)s ") % {
+                    "call_id": call_id, "fname": f.__name__}
+    end_message = (message_base + "took %(seconds).3fs seconds to run")
+
+    @timeutils.time_it(LOG, message=end_message, min_duration=None)
+    def wrapper(*args, **kwargs):
+        LOG.debug(message_base + "start")
+        ret = f(*args, **kwargs)
+        return ret
+    return wrapper
+
+
+class SingletonDecorator:
+
+    def __init__(self, klass):
+        self._klass = klass
+        self._instance = None
+
+    def __call__(self, *args, **kwargs):
+        if self._instance is None:
+            self._instance = self._klass(*args, **kwargs)
+        return self._instance
+
+
+def with_metaclass(meta, *bases):
+    """Function from jinja2/_compat.py. License: BSD.
+
+    Method imported from "futures".
+
+    Use it like this::
+
+        class BaseForm(object):
+            pass
+
+        class FormType(type):
+            pass
+
+        class Form(with_metaclass(FormType, BaseForm)):
+            pass
+
+    This requires a bit of explanation: the basic idea is to make a
+    dummy metaclass for one level of class instantiation that replaces
+    itself with the actual metaclass.  Because of internal type checks
+    we also need to make sure that we downgrade the custom metaclass
+    for one level to something closer to type (that's why __call__ and
+    __init__ comes back from type etc.).
+
+    This has the advantage over six.with_metaclass of not introducing
+    dummy classes into the final MRO.
+    """
+    class metaclass(meta):
+        __call__ = type.__call__
+        __init__ = type.__init__
+
+        def __new__(cls, name, this_bases, d):
+            if this_bases is None:
+                return type.__new__(cls, name, (), d)
+            return meta(name, bases, d)
+
+    return metaclass('temporary_class', None, {})
