@@ -13,124 +13,102 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import datetime
 
 from eventlet import greenthread
+from neutron_lib.agent import constants as agent_consts
 from neutron_lib.api import converters
+from neutron_lib.api.definitions import agent as agent_apidef
+from neutron_lib.api import extensions
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants
-from neutron_lib.db import model_base
+from neutron_lib import context
+from neutron_lib.db import api as db_api
+from neutron_lib.db import model_query
+from neutron_lib.db import utils as db_utils
+from neutron_lib.exceptions import agent as agent_exc
+from neutron_lib.exceptions import availability_zone as az_exc
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_serialization import jsonutils
 from oslo_utils import importutils
 from oslo_utils import timeutils
-import six
-import sqlalchemy as sa
-from sqlalchemy.orm import exc
-from sqlalchemy import sql
 
-from neutron._i18n import _, _LE, _LI, _LW
+from neutron.agent.common import utils
 from neutron.api.rpc.callbacks import version_manager
-from neutron.api.v2 import attributes
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.common import constants as n_const
-from neutron import context
-from neutron.db import api as db_api
+from neutron.conf.agent.database import agents_db
+from neutron.db.models import agent as agent_model
+from neutron.extensions import _availability_zone_filter_lib as azfil_ext
 from neutron.extensions import agent as ext_agent
 from neutron.extensions import availability_zone as az_ext
-from neutron import manager
+from neutron.objects import agent as agent_obj
+
 
 LOG = logging.getLogger(__name__)
 
-AGENT_OPTS = [
-    cfg.IntOpt('agent_down_time', default=75,
-               help=_("Seconds to regard the agent is down; should be at "
-                      "least twice report_interval, to be sure the "
-                      "agent is down for good.")),
-    cfg.StrOpt('dhcp_load_type', default='networks',
-               choices=['networks', 'subnets', 'ports'],
-               help=_('Representing the resource type whose load is being '
-                      'reported by the agent. This can be "networks", '
-                      '"subnets" or "ports". '
-                      'When specified (Default is networks), the server will '
-                      'extract particular load sent as part of its agent '
-                      'configuration object from the agent report state, '
-                      'which is the number of resources being consumed, at '
-                      'every report_interval.'
-                      'dhcp_load_type can be used in combination with '
-                      'network_scheduler_driver = '
-                      'neutron.scheduler.dhcp_agent_scheduler.WeightScheduler '
-                      'When the network_scheduler_driver is WeightScheduler, '
-                      'dhcp_load_type can be configured to represent the '
-                      'choice for the resource being balanced. '
-                      'Example: dhcp_load_type=networks')),
-    cfg.BoolOpt('enable_new_agents', default=True,
-                help=_("Agent starts with admin_state_up=False when "
-                       "enable_new_agents=False. In the case, user's "
-                       "resources will not be scheduled automatically to the "
-                       "agent until admin changes admin_state_up to True.")),
-]
-cfg.CONF.register_opts(AGENT_OPTS)
+agents_db.register_db_agents_opts()
 
 # this is the ratio from agent_down_time to the time we use to consider
 # the agents down for considering their resource versions in the
 # version_manager callback
 DOWNTIME_VERSIONS_RATIO = 2
 
+RESOURCE_AGENT_TYPE_MAP = {
+    'network': constants.AGENT_TYPE_DHCP,
+    'router': constants.AGENT_TYPE_L3,
+}
 
-class Agent(model_base.BASEV2, model_base.HasId):
-    """Represents agents running in neutron deployments."""
+AZ_ATTRIBUTE_MAP = {
+    'name': {
+        'agent_key': 'availability_zone',
+        'convert_to': lambda x: x,
+    },
+    'resource': {
+        'agent_key': 'agent_type',
+        'convert_to': lambda x: RESOURCE_AGENT_TYPE_MAP.get(x, x),
+    }
+}
 
-    __table_args__ = (
-        sa.UniqueConstraint('agent_type', 'host',
-                            name='uniq_agents0agent_type0host'),
-        model_base.BASEV2.__table_args__
-    )
 
-    # L3 agent, DHCP agent, OVS agent, LinuxBridge
-    agent_type = sa.Column(sa.String(255), nullable=False)
-    binary = sa.Column(sa.String(255), nullable=False)
-    # TOPIC is a fanout exchange topic
-    topic = sa.Column(sa.String(255), nullable=False)
-    # TOPIC.host is a target topic
-    host = sa.Column(sa.String(255), nullable=False)
-    availability_zone = sa.Column(sa.String(255))
-    admin_state_up = sa.Column(sa.Boolean, default=True,
-                               server_default=sql.true(), nullable=False)
-    # the time when first report came from agents
-    created_at = sa.Column(sa.DateTime, nullable=False)
-    # the time when first report came after agents start
-    started_at = sa.Column(sa.DateTime, nullable=False)
-    # updated when agents report
-    heartbeat_timestamp = sa.Column(sa.DateTime, nullable=False)
-    # description is note for admin user
-    description = sa.Column(sa.String(attributes.DESCRIPTION_MAX_LEN))
-    # configurations: a json dict string, I think 4095 is enough
-    configurations = sa.Column(sa.String(4095), nullable=False)
-    # resource_versions: json dict, 8191 allows for ~256 resource versions
-    #                    assuming ~32byte length "'name': 'ver',"
-    #                    the whole row limit is 65535 bytes in mysql
-    resource_versions = sa.Column(sa.String(8191))
-    # load - number of resources hosted by the agent
-    load = sa.Column(sa.Integer, server_default='0', nullable=False)
+def get_availability_zones_by_agent_type(context, agent_type,
+                                         availability_zones):
+    """Get list of availability zones based on agent type"""
 
-    @property
-    def is_active(self):
-        return not AgentDbMixin.is_agent_down(self.heartbeat_timestamp)
+    agents = agent_obj.Agent._get_agents_by_availability_zones_and_agent_type(
+        context, agent_type=agent_type, availability_zones=availability_zones)
+    return set(agent.availability_zone for agent in agents)
 
 
 class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
     """Mixin class to add availability_zone extension to AgentDbMixin."""
 
+    _is_az_filter_supported = None
+
+    @property
+    def is_az_filter_supported(self):
+        supported = self._is_az_filter_supported
+        if supported is None:
+            supported = False
+            for plugin in directory.get_plugins().values():
+                if extensions.is_extension_supported(plugin, azfil_ext.ALIAS):
+                    supported = True
+                    break
+        self._is_az_filter_supported = supported
+
+        return self._is_az_filter_supported
+
     def _list_availability_zones(self, context, filters=None):
         result = {}
-        query = self._get_collection_query(context, Agent, filters=filters)
-        for agent in query.group_by(Agent.admin_state_up,
-                                    Agent.availability_zone,
-                                    Agent.agent_type):
+        filters = filters or {}
+        if self._is_az_filter_supported or self.is_az_filter_supported:
+            filters = self._adjust_az_filters(filters)
+        agents = agent_obj.Agent.get_objects(context, **filters)
+        for agent in agents:
             if not agent.availability_zone:
                 continue
             if agent.agent_type == constants.AGENT_TYPE_DHCP:
@@ -140,21 +118,46 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
             else:
                 continue
             key = (agent.availability_zone, resource)
-            result[key] = agent.admin_state_up or result.get(key, False)
+            value = agent.admin_state_up or result.get(key, False)
+            result[key] = 'available' if value else 'unavailable'
         return result
 
+    def _adjust_az_filters(self, filters):
+        # The intersect of sets gets us applicable filter keys (others ignored)
+        common_keys = filters.keys() & AZ_ATTRIBUTE_MAP.keys()
+        for key in common_keys:
+            filter_key = AZ_ATTRIBUTE_MAP[key]['agent_key']
+            filter_vals = filters.pop(key)
+            if filter_vals:
+                filter_vals = [AZ_ATTRIBUTE_MAP[key]['convert_to'](v)
+                               for v in filter_vals]
+            filters.setdefault(filter_key, [])
+            filters[filter_key] += filter_vals
+        return filters
+
     @db_api.retry_if_session_inactive()
     def get_availability_zones(self, context, filters=None, fields=None,
                                sorts=None, limit=None, marker=None,
                                page_reverse=False):
         """Return a list of availability zones."""
-        # NOTE(hichihara): 'tenant_id' is dummy for policy check.
-        # it is not visible via API.
-        return [{'state': 'available' if v else 'unavailable',
-                 'name': k[0], 'resource': k[1],
-                 'tenant_id': context.tenant_id}
-                for k, v in six.iteritems(self._list_availability_zones(
-                                           context, filters))]
+        if self._is_az_filter_supported or self.is_az_filter_supported:
+            filter_states = filters.pop('state', [])
+            # NOTE(hichihara): 'tenant_id' is dummy for policy check.
+            # it is not visible via API.
+            return [{'state': v,
+                     'name': k[0], 'resource': k[1],
+                     'tenant_id': context.tenant_id}
+                    for k, v in self._list_availability_zones(
+                                               context, filters).items()
+                    if not filter_states or v in filter_states]
+        else:
+            # NOTE(hichihara): 'tenant_id' is dummy for policy check.
+            # it is not visible via API.
+            return [{'state': v,
+                     'name': k[0], 'resource': k[1],
+                     'tenant_id': context.tenant_id}
+                    for k, v in self._list_availability_zones(
+                                               context, filters).items()]
 
     @db_api.retry_if_session_inactive()
     def validate_availability_zones(self, context, resource_type,
@@ -168,48 +171,41 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
             agent_type = constants.AGENT_TYPE_L3
         else:
             return
-        query = context.session.query(Agent.availability_zone).filter_by(
-                    agent_type=agent_type).group_by(Agent.availability_zone)
-        query = query.filter(Agent.availability_zone.in_(availability_zones))
-        azs = [item[0] for item in query]
+        azs = get_availability_zones_by_agent_type(
+            context, agent_type, availability_zones)
         diff = set(availability_zones) - set(azs)
         if diff:
-            raise az_ext.AvailabilityZoneNotFound(availability_zone=diff.pop())
+            raise az_exc.AvailabilityZoneNotFound(availability_zone=diff.pop())
 
 
 class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
     """Mixin class to add agent extension to db_base_plugin_v2."""
 
     def _get_agent(self, context, id):
-        try:
-            agent = self._get_by_id(context, Agent, id)
-        except exc.NoResultFound:
-            raise ext_agent.AgentNotFound(id=id)
+        agent = agent_obj.Agent.get_object(context, id=id)
+        if not agent:
+            raise agent_exc.AgentNotFound(id=id)
         return agent
 
     @db_api.retry_if_session_inactive()
     def get_enabled_agent_on_host(self, context, agent_type, host):
         """Return agent of agent_type for the specified host."""
-        query = context.session.query(Agent)
-        query = query.filter(Agent.agent_type == agent_type,
-                             Agent.host == host,
-                             Agent.admin_state_up == sql.true())
-        try:
-            agent = query.one()
-        except exc.NoResultFound:
+
+        agent = agent_obj.Agent.get_object(context,
+                                           agent_type=agent_type,
+                                           host=host,
+                                           admin_state_up=True)
+
+        if not agent:
             LOG.debug('No enabled %(agent_type)s agent on host '
                       '%(host)s', {'agent_type': agent_type, 'host': host})
             return
-        if self.is_agent_down(agent.heartbeat_timestamp):
-            LOG.warning(_LW('%(agent_type)s agent %(agent_id)s is not active'),
+
+        if utils.is_agent_down(agent.heartbeat_timestamp):
+            LOG.warning('%(agent_type)s agent %(agent_id)s is not active',
                         {'agent_type': agent_type, 'agent_id': agent.id})
         return agent
 
-    @staticmethod
-    def is_agent_down(heart_beat_time):
-        return timeutils.is_older_than(heart_beat_time,
-                                       cfg.CONF.agent_down_time)
-
     @staticmethod
     def is_agent_considered_for_versions(agent_dict):
         return not timeutils.is_older_than(agent_dict['heartbeat_timestamp'],
@@ -223,11 +219,18 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
         json_value = None
         try:
             json_value = getattr(agent_db, dict_name)
-            conf = jsonutils.loads(json_value)
+            # TODO(tuanvu): after all agent_db is converted to agent_obj,
+            #               we no longer need this.
+            #               Without this check, some unit tests will fail
+            #               because some of json_values are dict already
+            if not isinstance(json_value, dict):
+                conf = jsonutils.loads(json_value)
+            else:
+                conf = json_value
         except Exception:
             if json_value or not ignore_missing:
-                msg = _LW('Dictionary %(dict_name)s for agent %(agent_type)s '
-                          'on host %(host)s is invalid.')
+                msg = ('Dictionary %(dict_name)s for agent %(agent_type)s '
+                       'on host %(host)s is invalid.')
                 LOG.warning(msg, {'dict_name': dict_name,
                                   'agent_type': agent_db.agent_type,
                                   'host': agent_db.host})
@@ -245,48 +248,61 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
         return load
 
     def _make_agent_dict(self, agent, fields=None):
-        attr = ext_agent.RESOURCE_ATTRIBUTE_MAP.get(
-            ext_agent.RESOURCE_NAME + 's')
+        attr = agent_apidef.RESOURCE_ATTRIBUTE_MAP.get(
+            agent_apidef.COLLECTION_NAME)
         res = dict((k, agent[k]) for k in attr
                    if k not in ['alive', 'configurations'])
-        res['alive'] = not self.is_agent_down(res['heartbeat_timestamp'])
+        res['alive'] = not utils.is_agent_down(
+            res['heartbeat_timestamp']
+        )
         res['configurations'] = self._get_dict(agent, 'configurations')
         res['resource_versions'] = self._get_dict(agent, 'resource_versions',
                                                   ignore_missing=True)
         res['availability_zone'] = agent['availability_zone']
-        return self._fields(res, fields)
+        res['resources_synced'] = agent['resources_synced']
+        return db_utils.resource_fields(res, fields)
 
     @db_api.retry_if_session_inactive()
     def delete_agent(self, context, id):
         agent = self._get_agent(context, id)
-        registry.notify(resources.AGENT, events.BEFORE_DELETE, self,
-                        context=context, agent=agent)
-        with context.session.begin(subtransactions=True):
-            context.session.delete(agent)
+        registry.publish(resources.AGENT, events.BEFORE_DELETE, self,
+                         payload=events.DBEventPayload(
+                             context, states=(agent,), resource_id=id))
+        agent.delete()
 
     @db_api.retry_if_session_inactive()
     def update_agent(self, context, id, agent):
         agent_data = agent['agent']
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             agent = self._get_agent(context, id)
-            agent.update(agent_data)
+            agent.update_fields(agent_data)
+            agent.update()
         return self._make_agent_dict(agent)
 
     @db_api.retry_if_session_inactive()
     def get_agents_db(self, context, filters=None):
-        query = self._get_collection_query(context, Agent, filters=filters)
+        # TODO(annp): keep this method for backward compatibility,
+        #             will need to clean it up later
+        query = model_query.get_collection_query(context,
+                                                 agent_model.Agent,
+                                                 filters=filters)
         return query.all()
 
+    @db_api.retry_if_session_inactive()
+    def get_agent_objects(self, context, filters=None):
+        filters = filters or {}
+        return agent_obj.Agent.get_objects(context, **filters)
+
     @db_api.retry_if_session_inactive()
     def get_agents(self, context, filters=None, fields=None):
-        agents = self._get_collection(context, Agent,
-                                      self._make_agent_dict,
-                                      filters=filters, fields=fields)
-        alive = filters and filters.get('alive', None)
+        filters = filters or {}
+        alive = filters and filters.pop('alive', None)
+        agents = agent_obj.Agent.get_objects(context, **filters)
         if alive:
             alive = converters.convert_to_boolean(alive[0])
-            agents = [agent for agent in agents if agent['alive'] == alive]
-        return agents
+            agents = [agent for agent in agents if agent.is_active == alive]
+        return [self._make_agent_dict(agent, fields=fields)
+                for agent in agents]
 
     @db_api.retry_db_errors
     def agent_health_check(self):
@@ -300,8 +316,8 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                                (agent['agent_type'],
                                 agent['heartbeat_timestamp'],
                                 agent['host']) for agent in dead_agents])
-            LOG.warning(_LW("Agent healthcheck: found %(count)s dead agents "
-                            "out of %(total)s:\n%(data)s"),
+            LOG.warning("Agent healthcheck: found %(count)s dead agents "
+                        "out of %(total)s:\n%(data)s",
                         {'count': len(dead_agents),
                          'total': len(agents),
                          'data': data})
@@ -310,17 +326,16 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                       len(agents))
 
     def _get_agent_by_type_and_host(self, context, agent_type, host):
-        query = self._model_query(context, Agent)
-        try:
-            agent_db = query.filter(Agent.agent_type == agent_type,
-                                    Agent.host == host).one()
-            return agent_db
-        except exc.NoResultFound:
-            raise ext_agent.AgentNotFoundByTypeHost(agent_type=agent_type,
+        agent_objs = agent_obj.Agent.get_objects(context,
+                                                 agent_type=agent_type,
+                                                 host=host)
+        if not agent_objs:
+            raise agent_exc.AgentNotFoundByTypeHost(agent_type=agent_type,
                                                     host=host)
-        except exc.MultipleResultsFound:
-            raise ext_agent.MultipleAgentFoundByTypeHost(agent_type=agent_type,
-                                                         host=host)
+        if len(agent_objs) > 1:
+            raise agent_exc.MultipleAgentFoundByTypeHost(
+                agent_type=agent_type, host=host)
+        return agent_objs[0]
 
     @db_api.retry_if_session_inactive()
     def get_agent(self, context, id, fields=None):
@@ -340,26 +355,29 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
         """
         return candidate_hosts
 
-    def _log_heartbeat(self, state, agent_db, agent_conf):
+    def _log_heartbeat(self, state, agent_db, agent_conf, agent_timestamp):
         if agent_conf.get('log_agent_heartbeats'):
             delta = timeutils.utcnow() - agent_db.heartbeat_timestamp
-            LOG.info(_LI("Heartbeat received from %(type)s agent on "
-                         "host %(host)s, uuid %(uuid)s after %(delta)s"),
+            LOG.info("Heartbeat received from %(type)s agent on "
+                     "host %(host)s, uuid %(uuid)s after %(delta)s, sent at "
+                     "%(agent_timestamp)s",
                      {'type': agent_db.agent_type,
                       'host': agent_db.host,
                       'uuid': state.get('uuid'),
-                      'delta': delta})
+                      'delta': delta,
+                      'agent_timestamp': agent_timestamp})
 
     @db_api.retry_if_session_inactive()
-    def create_or_update_agent(self, context, agent_state):
+    def create_or_update_agent(self, context, agent_state,
+                               agent_timestamp=None):
         """Registers new agent in the database or updates existing.
 
         Returns tuple of agent status and state.
         Status is from server point of view: alive, new or revived.
         It could be used by agent to do some sync with the server if needed.
         """
-        status = n_const.AGENT_ALIVE
-        with context.session.begin(subtransactions=True):
+        status = agent_consts.AGENT_ALIVE
+        with db_api.CONTEXT_WRITER.using(context):
             res_keys = ['agent_type', 'binary', 'host', 'topic']
             res = dict((k, agent_state[k]) for k in res_keys)
             if 'availability_zone' in agent_state:
@@ -373,41 +391,57 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
             res['load'] = self._get_agent_load(agent_state)
             current_time = timeutils.utcnow()
             try:
-                agent_db = self._get_agent_by_type_and_host(
+                agent = self._get_agent_by_type_and_host(
                     context, agent_state['agent_type'], agent_state['host'])
-                if not agent_db.is_active:
-                    status = n_const.AGENT_REVIVED
+                agent_state_orig = copy.deepcopy(agent_state)
+                if not agent.is_active:
+                    status = agent_consts.AGENT_REVIVED
                     if 'resource_versions' not in agent_state:
                         # updating agent_state with resource_versions taken
                         # from db so that
                         # _update_local_agent_resource_versions() will call
                         # version_manager and bring it up to date
                         agent_state['resource_versions'] = self._get_dict(
-                            agent_db, 'resource_versions', ignore_missing=True)
+                            agent, 'resource_versions', ignore_missing=True)
                 res['heartbeat_timestamp'] = current_time
                 if agent_state.get('start_flag'):
                     res['started_at'] = current_time
                 greenthread.sleep(0)
-                self._log_heartbeat(agent_state, agent_db, configurations_dict)
-                agent_db.update(res)
+                self._log_heartbeat(agent_state, agent, configurations_dict,
+                                    agent_timestamp)
+                agent.update_fields(res)
+                agent.update()
                 event_type = events.AFTER_UPDATE
-            except ext_agent.AgentNotFoundByTypeHost:
+            except agent_exc.AgentNotFoundByTypeHost:
+                agent_state_orig = None
                 greenthread.sleep(0)
                 res['created_at'] = current_time
                 res['started_at'] = current_time
                 res['heartbeat_timestamp'] = current_time
                 res['admin_state_up'] = cfg.CONF.enable_new_agents
-                agent_db = Agent(**res)
+                agent = agent_obj.Agent(context=context, **res)
                 greenthread.sleep(0)
-                context.session.add(agent_db)
+                agent.create()
                 event_type = events.AFTER_CREATE
-                self._log_heartbeat(agent_state, agent_db, configurations_dict)
-                status = n_const.AGENT_NEW
+                self._log_heartbeat(agent_state, agent, configurations_dict,
+                                    agent_timestamp)
+                status = agent_consts.AGENT_NEW
             greenthread.sleep(0)
 
-        registry.notify(resources.AGENT, event_type, self, context=context,
-                        host=agent_state['host'], plugin=self,
-                        agent=agent_state)
+        agent_state['agent_status'] = status
+        agent_state['admin_state_up'] = agent.admin_state_up
+        agent_state['id'] = agent.id
+        registry.publish(resources.AGENT, event_type, self,
+                         payload=events.DBEventPayload(
+                             context=context, metadata={
+                                 'host': agent_state['host'],
+                                 'plugin': self,
+                                 'status': status
+                             },
+                             states=(agent_state_orig, ),
+                             desired_state=agent_state,
+                             resource_id=agent.id
+                         ))
         return status, agent_state
 
     def _get_agents_considered_for_versions(self):
@@ -429,9 +463,6 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
             resource_versions = agent.get('resource_versions', {})
             consumer = version_manager.AgentConsumer(
                 agent_type=agent['agent_type'], host=agent['host'])
-            LOG.debug("Update consumer %(consumer)s versions to: "
-                      "%(versions)s", {'consumer': consumer,
-                                       'versions': resource_versions})
             tracker.set_versions(consumer, resource_versions)
 
 
@@ -440,27 +471,32 @@ class AgentExtRpcCallback(object):
 
     This class implements the server side of an rpc interface.  The client side
     can be found in neutron.agent.rpc.PluginReportStateAPI.  For more
-    information on changing rpc interfaces, see doc/source/devref/rpc_api.rst.
+    information on changing rpc interfaces, see
+    doc/source/contributor/internals/rpc_api.rst.
 
     API version history:
         1.0 - Initial version.
         1.1 - report_state now returns agent state.
+        1.2 - add method has_alive_neutron_server.
     """
 
-    target = oslo_messaging.Target(version='1.1',
-                                   namespace=n_const.RPC_NAMESPACE_STATE)
+    target = oslo_messaging.Target(version='1.2',
+                                   namespace=constants.RPC_NAMESPACE_STATE)
     START_TIME = timeutils.utcnow()
 
     def __init__(self, plugin=None):
         super(AgentExtRpcCallback, self).__init__()
         self.plugin = plugin
-        #TODO(ajo): fix the resources circular dependency issue by dynamically
-        #           registering object types in the RPC callbacks api
+        # TODO(ajo): fix the resources circular dependency issue by dynamically
+        #            registering object types in the RPC callbacks api
         resources_rpc = importutils.import_module(
             'neutron.api.rpc.handlers.resources_rpc')
         # Initialize RPC api directed to other neutron-servers
         self.server_versions_rpc = resources_rpc.ResourcesPushToServersRpcApi()
 
+    def has_alive_neutron_server(self, context, **kwargs):
+        return True
+
     @db_api.retry_if_session_inactive()
     def report_state(self, context, **kwargs):
         """Report state from agent to server.
@@ -480,9 +516,9 @@ class AgentExtRpcCallback(object):
                       "server start timestamp: %(server_time)s", log_dict)
             return
         if not self.plugin:
-            self.plugin = manager.NeutronManager.get_plugin()
+            self.plugin = directory.get_plugin()
         agent_status, agent_state = self.plugin.create_or_update_agent(
-            context, agent_state)
+            context, agent_state, time)
         self._update_local_agent_resource_versions(context, agent_state)
         return agent_status
 
@@ -521,10 +557,10 @@ class AgentExtRpcCallback(object):
                             'serv_time': (datetime.datetime.isoformat
                                           (time_server_now)),
                             'diff': diff}
-                LOG.error(_LE("Message received from the host: %(host)s "
-                              "during the registration of %(agent_name)s has "
-                              "a timestamp: %(agent_time)s. This differs from "
-                              "the current server timestamp: %(serv_time)s by "
-                              "%(diff)s seconds, which is more than the "
-                              "threshold agent down"
-                              "time: %(threshold)s."), log_dict)
+                LOG.error("Message received from the host: %(host)s "
+                          "during the registration of %(agent_name)s has "
+                          "a timestamp: %(agent_time)s. This differs from "
+                          "the current server timestamp: %(serv_time)s by "
+                          "%(diff)s seconds, which is more than the "
+                          "threshold agent down "
+                          "time: %(threshold)s.", log_dict)
