@@ -17,8 +17,9 @@
 from oslo_log import log as logging
 from oslo_utils import excutils
 
-from neutron._i18n import _LI
+from neutron._i18n import _
 from neutron.agent.common import ovs_lib
+from neutron.common import ipv6_utils
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
         as ovs_consts
 from neutron.plugins.ml2.drivers.openvswitch.agent.openflow \
@@ -42,42 +43,38 @@ class OVSAgentBridge(ofswitch.OpenFlowSwitchMixin,
         A convenient method for openflow message composers.
         """
         while True:
-            dpid_int = self._cached_dpid
-            if dpid_int is None:
-                dpid_str = self.get_datapath_id()
-                LOG.info(_LI("Bridge %(br_name)s has datapath-ID %(dpid)s"),
-                         {"br_name": self.br_name, "dpid": dpid_str})
-                dpid_int = int(dpid_str, 16)
+            if self._cached_dpid is None:
+                dpid = self.get_datapath_id()
+                LOG.info("Bridge %(br_name)s has datapath-ID %(dpid)s",
+                         {"br_name": self.br_name, "dpid": dpid})
+                if dpid is None:
+                    raise RuntimeError(_("Unknown datapath id."))
+                self._cached_dpid = int(dpid, 16)
             try:
-                dp = self._get_dp_by_dpid(dpid_int)
+                dp = self._get_dp_by_dpid(self._cached_dpid)
+                return dp, dp.ofproto, dp.ofproto_parser
             except RuntimeError:
                 with excutils.save_and_reraise_exception() as ctx:
-                    self._cached_dpid = None
                     # Retry if dpid has been changed.
                     # NOTE(yamamoto): Open vSwitch change its dpid on
                     # some events.
                     # REVISIT(yamamoto): Consider to set dpid statically.
-                    new_dpid_str = self.get_datapath_id()
-                    if new_dpid_str != dpid_str:
-                        LOG.info(_LI("Bridge %(br_name)s changed its "
-                                     "datapath-ID from %(old)s to %(new)s"), {
-                            "br_name": self.br_name,
-                            "old": dpid_str,
-                            "new": new_dpid_str,
-                        })
+                    new_dpid = int(self.get_datapath_id(), 16)
+                    if new_dpid != self._cached_dpid:
+                        LOG.info("Bridge %(br_name)s changed its "
+                                 "datapath-ID from %(old)x to %(new)x", {
+                                     "br_name": self.br_name,
+                                     "old": self._cached_dpid,
+                                     "new": new_dpid,
+                                 })
                         ctx.reraise = False
-            else:
-                self._cached_dpid = dpid_int
-                return dp, dp.ofproto, dp.ofproto_parser
+                    self._cached_dpid = new_dpid
 
     def setup_controllers(self, conf):
-        controllers = [
-            "tcp:%(address)s:%(port)s" % {
-                "address": conf.OVS.of_listen_address,
-                "port": conf.OVS.of_listen_port,
-            }
-        ]
-        self.set_protocols(ovs_consts.OPENFLOW13)
+        url = ipv6_utils.valid_ipv6_url(conf.OVS.of_listen_address,
+                                        conf.OVS.of_listen_port)
+        controllers = ["tcp:" + url]
+        self.add_protocols(ovs_consts.OPENFLOW13)
         self.set_controller(controllers)
 
         # NOTE(ivc): Force "out-of-band" controller connection mode (see
@@ -95,6 +92,7 @@ class OVSAgentBridge(ofswitch.OpenFlowSwitchMixin,
         #
         # [1] https://github.com/openvswitch/ovs/blob/master/DESIGN.md
         self.set_controllers_connection_mode("out-of-band")
+        self.set_controllers_inactivity_probe(conf.OVS.of_inactivity_probe)
 
     def drop_port(self, in_port):
         self.install_drop(priority=2, in_port=in_port)
