@@ -485,4 +485,4 @@ class BaseOVSTestCase(base.BaseSudoTestCase):
                 ipv6_port_options = interface['options']
         self.assertEqual(p_const.TYPE_GRE, ipv4_port_type)
         self.assertEqual(ovs_lib.TYPE_GRE_IP6, ipv6_port_type)
-        self.assertEqual('legacy', ipv6_port_options.get('packet_type'))
+        self.assertEqual('legacy_l2', ipv6_port_options.get('packet_type'))
