@@ -13,18 +13,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import copy
 import datetime
-import mock
+from unittest import mock
 
+from neutron_lib.api import extensions
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
+from neutron_lib import constants
+from neutron_lib.plugins import directory
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 
 from neutron.api.rpc.agentnotifiers import dhcp_rpc_agent_api
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.common import utils
-from neutron.db import agents_db
 from neutron.db.agentschedulers_db import cfg
+from neutron.objects import agent as agent_obj
 from neutron.tests import base
 
 
@@ -35,7 +39,7 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
         self.notifier = (
             dhcp_rpc_agent_api.DhcpAgentNotifyAPI(plugin=mock.Mock()))
 
-        mock_util_p = mock.patch.object(utils, 'is_extension_supported')
+        mock_util_p = mock.patch.object(extensions, 'is_extension_supported')
         mock_log_p = mock.patch.object(dhcp_rpc_agent_api, 'LOG')
         mock_fanout_p = mock.patch.object(self.notifier, '_fanout_message')
         mock_cast_p = mock.patch.object(self.notifier, '_cast_message')
@@ -57,7 +61,8 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
         self.assertEqual(expected_warnings, self.mock_log.warning.call_count)
 
     def test__schedule_network(self):
-        agent = agents_db.Agent()
+        agent = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid(),
+                                host='host')
         agent.admin_state_up = True
         agent.heartbeat_timestamp = timeutils.utcnow()
         network = {'id': 'foo_net_id'}
@@ -66,7 +71,7 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
                                      expected_casts=1, expected_warnings=0)
 
     def test__schedule_network_no_existing_agents(self):
-        agent = agents_db.Agent()
+        agent = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent.admin_state_up = True
         agent.heartbeat_timestamp = timeutils.utcnow()
         network = {'id': 'foo_net_id'}
@@ -93,20 +98,20 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
         self.assertEqual(expected_errors, self.mock_log.error.call_count)
 
     def test__get_enabled_agents(self):
-        agent1 = agents_db.Agent()
+        agent1 = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent1.admin_state_up = True
         agent1.heartbeat_timestamp = timeutils.utcnow()
-        agent2 = agents_db.Agent()
+        agent2 = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent2.admin_state_up = False
         agent2.heartbeat_timestamp = timeutils.utcnow()
         network = {'id': 'foo_network_id'}
         self._test__get_enabled_agents(network, agents=[agent1])
 
     def test__get_enabled_agents_with_inactive_ones(self):
-        agent1 = agents_db.Agent()
+        agent1 = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent1.admin_state_up = True
         agent1.heartbeat_timestamp = timeutils.utcnow()
-        agent2 = agents_db.Agent()
+        agent2 = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent2.admin_state_up = True
         # This is effectively an inactive agent
         agent2.heartbeat_timestamp = datetime.datetime(2000, 1, 1, 0, 0)
@@ -117,7 +122,7 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
 
     def test__get_enabled_agents_with_notification_required(self):
         network = {'id': 'foo_network_id', 'subnets': ['foo_subnet_id']}
-        agent = agents_db.Agent()
+        agent = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent.admin_state_up = False
         agent.heartbeat_timestamp = timeutils.utcnow()
         self._test__get_enabled_agents(network, [agent], port_count=20,
@@ -126,26 +131,61 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
     def test__get_enabled_agents_with_admin_state_down(self):
         cfg.CONF.set_override(
             'enable_services_on_agents_with_admin_state_down', True)
-        agent1 = agents_db.Agent()
+        agent1 = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent1.admin_state_up = True
         agent1.heartbeat_timestamp = timeutils.utcnow()
-        agent2 = agents_db.Agent()
+        agent2 = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid())
         agent2.admin_state_up = False
         agent2.heartbeat_timestamp = timeutils.utcnow()
         network = {'id': 'foo_network_id'}
         self._test__get_enabled_agents(network, agents=[agent1, agent2])
 
+    def test__notify_agents_allocate_priority(self):
+        mock_context = mock.MagicMock()
+        mock_context.is_admin = True
+        methods = ['network_create_end', 'network_update_end',
+                  'network_delete_end', 'subnet_create_end',
+                  'subnet_update_end', 'subnet_delete_end',
+                  'port_create_end', 'port_update_end', 'port_delete_end']
+        with mock.patch.object(self.notifier, '_schedule_network') as f:
+            with mock.patch.object(self.notifier, '_get_enabled_agents') as g:
+                for method in methods:
+                    f.return_value = [mock.MagicMock()]
+                    g.return_value = [mock.MagicMock()]
+                    payload = {}
+                    if method.startswith('port'):
+                        payload['port'] = \
+                            {'device_id':
+                             constants.DEVICE_ID_RESERVED_DHCP_PORT}
+                    expected_payload = copy.deepcopy(payload)
+                    expected_payload['priority'] = \
+                        dhcp_rpc_agent_api.METHOD_PRIORITY_MAP.get(method)
+                    self.notifier._notify_agents(mock_context, method, payload,
+                                                 'fake_network_id')
+                    if method == 'network_delete_end':
+                        self.mock_fanout.assert_called_with(mock.ANY, method,
+                                                            expected_payload)
+                    elif method != 'network_create_end':
+                        if method == 'port_create_end':
+                            expected_payload['priority'] = \
+                                dhcp_rpc_agent_api.PRIORITY_PORT_CREATE_HIGH
+                        self.mock_cast.assert_called_with(mock.ANY, method,
+                                                          expected_payload,
+                                                          mock.ANY, mock.ANY)
+
     def test__notify_agents_fanout_required(self):
         self.notifier._notify_agents(mock.ANY,
                                      'network_delete_end',
-                                     mock.ANY, 'foo_network_id')
+                                     {}, 'foo_network_id')
         self.assertEqual(1, self.mock_fanout.call_count)
 
-    def _test__notify_agents_with_function(
-        self, function, expected_scheduling=0, expected_casts=0):
+    def _test__notify_agents_with_function(self, function,
+                                           expected_scheduling=0,
+                                           expected_casts=0):
         with mock.patch.object(self.notifier, '_schedule_network') as f:
             with mock.patch.object(self.notifier, '_get_enabled_agents') as g:
-                agent = agents_db.Agent()
+                agent = agent_obj.Agent(mock.ANY, id=uuidutils.generate_uuid(),
+                                        host='host', topic='topic')
                 agent.admin_state_up = True
                 agent.heartbeat_timestamp = timeutils.utcnow()
                 g.return_value = [agent]
@@ -184,11 +224,10 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
         self.notifier.plugin.get_network.return_value = {'id': network_id}
         segment_sp = mock.Mock()
         segment_sp.get_segment.return_value = segment
-        with mock.patch('neutron.manager.NeutronManager.get_service_plugins',
-                        return_value={'segments': segment_sp}):
-            self._test__notify_agents('subnet_create_end',
-                                      expected_scheduling=1, expected_casts=1,
-                                      payload=subnet)
+        directory.add_plugin('segments', segment_sp)
+        self._test__notify_agents('subnet_create_end',
+                                  expected_scheduling=1, expected_casts=1,
+                                  payload=subnet)
         get_agents = self.notifier.plugin.get_dhcp_agents_hosting_networks
         get_agents.assert_called_once_with(
             mock.ANY, [network_id], hosts=segment['hosts'])
@@ -208,7 +247,9 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
         self._test__notify_agents_with_function(
             lambda: self.notifier._after_router_interface_deleted(
                 mock.ANY, mock.ANY, mock.ANY, context=mock.Mock(),
-                port={'id': 'foo_port_id', 'network_id': 'foo_network_id'}),
+                port={'id': 'foo_port_id', 'network_id': 'foo_network_id',
+                      'fixed_ips': {'subnet_id': 'subnet1',
+                                    'ip_address': '10.0.0.1'}}),
             expected_scheduling=0, expected_casts=1)
 
     def test__fanout_message(self):
@@ -216,7 +257,7 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
         self.assertEqual(1, self.mock_fanout.call_count)
 
     def test__cast_message(self):
-        self.notifier._cast_message(mock.ANY, mock.ANY, mock.ANY)
+        self.notifier._cast_message(mock.ANY, mock.ANY, mock.ANY, mock.ANY)
         self.assertEqual(1, self.mock_cast.call_count)
 
     def test__native_notification_unsubscribes(self):
@@ -238,3 +279,18 @@ class TestDhcpAgentNotifyAPI(base.BaseTestCase):
             registry.notify(res, events.AFTER_CREATE, self,
                             context=mock.Mock(), **kwargs)
             self.assertEqual([res], self.notifier._unsubscribed_resources)
+
+    def test__only_status_changed(self):
+        p1 = {'id': 1, 'status': 'DOWN', 'updated_at': '10:00:00',
+              'revision_number': 1}
+        p2 = dict(p1)
+        p2['status'] = 'ACTIVE'
+        p2['revision_number'] = 2
+        p2['updated_at'] = '10:00:01'
+        self.assertTrue(self.notifier._only_status_changed(p1, p2))
+        p2['name'] = 'test'
+        self.assertFalse(self.notifier._only_status_changed(p1, p2))
+        p1['name'] = 'test'
+        self.assertTrue(self.notifier._only_status_changed(p1, p2))
+        p1['name'] = 'test1'
+        self.assertFalse(self.notifier._only_status_changed(p1, p2))
