@@ -14,17 +14,16 @@
 #    under the License.
 #
 
+import collections
 import time
+from unittest import mock
 
-import mock
 from neutron_lib import constants as n_const
 from oslo_config import cfg
 from oslo_log import log
-import six
 
 from neutron.agent.common import ip_lib
 from neutron.agent.common import ovs_lib
-from neutron.plugins.common import constants as p_const
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent \
     import ovs_test_base
@@ -32,11 +31,7 @@ from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent \
     import test_vlanmanager
 
 
-def nonzero(f):
-    if six.PY3:
-        return f.__bool__()
-    else:
-        return f.__nonzero__()
+Switch = collections.namedtuple('Switch', ['br_name'])
 
 # Useful global dummy variables.
 NET_UUID = '3faeebfe-5d37-11e1-a64b-000c29d5f0a7'
@@ -46,14 +41,14 @@ LV_IDS = [42, 43]
 VIF_ID = '404deaec-5d37-11e1-a64b-000c29d5f0a8'
 VIF_MAC = '3c:09:24:1e:78:23'
 OFPORT_NUM = 1
-VIF_PORT = ovs_lib.VifPort('port', OFPORT_NUM,
-                           VIF_ID, VIF_MAC, 'switch')
+VIF_PORT = ovs_lib.VifPort('port', OFPORT_NUM, VIF_ID, VIF_MAC,
+                           Switch(br_name='br_name'))
 VIF_PORTS = {VIF_ID: VIF_PORT}
 FIXED_IPS = [{'subnet_id': 'my-subnet-uuid',
               'ip_address': '1.1.1.1'}]
 VM_DEVICE_OWNER = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'fake'
 
-TUN_OFPORTS = {p_const.TYPE_GRE: {'ip1': '11', 'ip2': '12'}}
+TUN_OFPORTS = {n_const.TYPE_GRE: {'ip1': '11', 'ip2': '12'}}
 
 BCAST_MAC = "01:00:00:00:00:00/01:00:00:00:00:00"
 UCAST_MAC = "00:00:00:00:00:00/01:00:00:00:00:00"
@@ -78,13 +73,17 @@ class TunnelTest(object):
         super(TunnelTest, self).setUp()
         self.useFixture(test_vlanmanager.LocalVlanManagerFixture())
         conn_patcher = mock.patch(
-            'neutron.agent.ovsdb.native.connection.Connection.start')
+            'neutron.agent.ovsdb.impl_idl._connection')
         conn_patcher.start()
+        mock.patch(
+            'neutron.api.rpc.handlers.resources_rpc.ResourcesPullRpcApi'
+        ).start()
         self.addCleanup(conn_patcher.stop)
         cfg.CONF.set_default('firewall_driver',
                              'neutron.agent.firewall.NoopFirewallDriver',
                              group='SECURITYGROUP')
         cfg.CONF.set_override('report_interval', 0, 'AGENT')
+        cfg.CONF.set_override('explicitly_egress_direct', True, 'AGENT')
 
         self.INT_BRIDGE = 'integration_bridge'
         self.TUN_BRIDGE = 'tunnel_bridge'
@@ -110,6 +109,7 @@ class TunnelTest(object):
                           new_callable=mock.PropertyMock,
                           return_value={}).start()
 
+        mock.patch('neutron.agent.ovsdb.impl_idl._connection').start()
         self.ovs_bridges = {
             self.INT_BRIDGE: mock.create_autospec(
                 self.br_int_cls('br-int')),
@@ -125,6 +125,9 @@ class TunnelTest(object):
             'int-%s' % self.MAP_TUN_BRIDGE: self.MAP_TUN_INT_OFPORT
         }
 
+        mock.patch('neutron.agent.rpc.PluginReportStateAPI.'
+                   'has_alive_neutron_server').start()
+
         def lookup_br(br_name, *args, **kwargs):
             return self.ovs_bridges[br_name]
 
@@ -179,13 +182,16 @@ class TunnelTest(object):
             ovs_lib.BaseOVS,
             'get_bridge_external_bridge_id').start()
         self.get_bridge_external_bridge_id.side_effect = (
-            lambda bridge: bridge if bridge in self.ovs_bridges else None)
+            lambda bridge, log_errors: bridge if bridge in self.ovs_bridges
+            else None)
 
         self.execute = mock.patch('neutron.agent.common.utils.execute').start()
-
+        self.mock_check_bridge_datapath_id = mock.patch.object(
+            self.mod_agent.OVSNeutronAgent,
+            '_check_bridge_datapath_id').start()
         self._define_expected_calls()
 
-    def _define_expected_calls(self, arp_responder=False):
+    def _define_expected_calls(self, arp_responder=False, igmp_snooping=False):
         self.mock_int_bridge_cls_expected = [
             mock.call(self.INT_BRIDGE,
                       datapath_type=mock.ANY),
@@ -204,6 +210,7 @@ class TunnelTest(object):
             mock.call.create(),
             mock.call.set_secure_mode(),
             mock.call.setup_controllers(mock.ANY),
+            mock.call.set_igmp_snooping_state(igmp_snooping),
             mock.call.setup_default_table(),
         ]
 
@@ -245,7 +252,7 @@ class TunnelTest(object):
             mock.call.create(secure_mode=True),
             mock.call.setup_controllers(mock.ANY),
             mock.call.port_exists('patch-int'),
-            nonzero(mock.call.port_exists()),
+            mock.ANY,
             mock.call.add_patch_port('patch-int', 'patch-tun'),
         ]
         self.mock_int_bridge_expected += [
@@ -272,6 +279,20 @@ class TunnelTest(object):
         self.intb_expected = []
         self.execute_expected = []
 
+        self.mock_int_bridge_expected += [
+            mock.call.install_goto(
+                dest_table_id=constants.LOCAL_MAC_DIRECT,
+                in_port=self.MAP_TUN_INT_OFPORT,
+                priority=4, table_id=constants.TRANSIENT_TABLE),
+            mock.call.install_goto(
+                dest_table_id=constants.LOCAL_MAC_DIRECT,
+                in_port=self.TUN_OFPORT,
+                priority=4, table_id=constants.TRANSIENT_TABLE),
+            mock.call.install_goto(
+                dest_table_id=constants.TRANSIENT_EGRESS_TABLE,
+                table_id=constants.LOCAL_MAC_DIRECT),
+        ]
+
     def _build_agent(self, **config_opts_agent):
         """Configure and initialize OVS agent.
 
@@ -297,7 +318,11 @@ class TunnelTest(object):
         for k, v in config_opts_agent.items():
             cfg.CONF.set_override(k, v, 'AGENT')
 
-        return self.mod_agent.OVSNeutronAgent(bridge_classes, cfg.CONF)
+        ext_mgr = mock.Mock()
+        agent = self.mod_agent.OVSNeutronAgent(
+            bridge_classes, ext_mgr, cfg.CONF)
+        mock.patch.object(agent.ovs.ovsdb, 'idl_monitor').start()
+        return agent
 
     def _verify_mock_call(self, mock_obj, expected):
         mock_obj.assert_has_calls(expected)
@@ -335,7 +360,13 @@ class TunnelTest(object):
     #                 The next two tests use l2_pop flag to test ARP responder
     def test_construct_with_arp_responder(self):
         self._build_agent(l2_population=True, arp_responder=True)
-        self._define_expected_calls(True)
+        self._define_expected_calls(arp_responder=True)
+        self._verify_mock_calls()
+
+    def test_construct_with_igmp_snooping(self):
+        cfg.CONF.set_override('igmp_snooping_enable', True, 'OVS')
+        self._build_agent()
+        self._define_expected_calls(igmp_snooping=True)
         self._verify_mock_calls()
 
     def test_construct_without_arp_responder(self):
@@ -347,11 +378,11 @@ class TunnelTest(object):
         self._verify_mock_calls()
 
     def test_provision_local_vlan(self):
-        ofports = list(TUN_OFPORTS[p_const.TYPE_GRE].values())
+        ofports = list(TUN_OFPORTS[n_const.TYPE_GRE].values())
         self.mock_tun_bridge_expected += [
             mock.call.install_flood_to_tun(LV_ID, LS_ID, ofports),
             mock.call.provision_local_vlan(
-                network_type=p_const.TYPE_GRE,
+                network_type=n_const.TYPE_GRE,
                 lvid=LV_ID,
                 segmentation_id=LS_ID),
         ]
@@ -359,7 +390,7 @@ class TunnelTest(object):
         a = self._build_agent()
         a.available_local_vlans = set([LV_ID])
         a.tun_br_ofports = TUN_OFPORTS
-        a.provision_local_vlan(NET_UUID, p_const.TYPE_GRE, None, LS_ID)
+        a.provision_local_vlan(NET_UUID, n_const.TYPE_GRE, None, LS_ID)
         self._verify_mock_calls()
 
     def test_provision_local_vlan_flat(self):
@@ -380,12 +411,12 @@ class TunnelTest(object):
         a.phys_brs['net1'] = self.mock_map_tun_bridge
         a.phys_ofports['net1'] = self.MAP_TUN_PHY_OFPORT
         a.int_ofports['net1'] = self.INT_OFPORT
-        a.provision_local_vlan(NET_UUID, p_const.TYPE_FLAT, 'net1', LS_ID)
+        a.provision_local_vlan(NET_UUID, n_const.TYPE_FLAT, 'net1', LS_ID)
         self._verify_mock_calls()
 
     def test_provision_local_vlan_flat_fail(self):
         a = self._build_agent()
-        a.provision_local_vlan(NET_UUID, p_const.TYPE_FLAT, 'net2', LS_ID)
+        a.provision_local_vlan(NET_UUID, n_const.TYPE_FLAT, 'net2', LS_ID)
         self._verify_mock_calls()
 
     def test_provision_local_vlan_vlan(self):
@@ -405,12 +436,12 @@ class TunnelTest(object):
         a.phys_brs['net1'] = self.mock_map_tun_bridge
         a.phys_ofports['net1'] = self.MAP_TUN_PHY_OFPORT
         a.int_ofports['net1'] = self.INT_OFPORT
-        a.provision_local_vlan(NET_UUID, p_const.TYPE_VLAN, 'net1', LS_ID)
+        a.provision_local_vlan(NET_UUID, n_const.TYPE_VLAN, 'net1', LS_ID)
         self._verify_mock_calls()
 
     def test_provision_local_vlan_vlan_fail(self):
         a = self._build_agent()
-        a.provision_local_vlan(NET_UUID, p_const.TYPE_VLAN, 'net2', LS_ID)
+        a.provision_local_vlan(NET_UUID, n_const.TYPE_VLAN, 'net2', LS_ID)
         self._verify_mock_calls()
 
     def test_reclaim_local_vlan(self):
@@ -470,8 +501,8 @@ class TunnelTest(object):
         self._verify_mock_calls()
 
     def test_port_bound(self):
-        vlan_mapping = {'segmentation_id': LS_ID,
-                        'physical_network': None,
+        vlan_mapping = {'segmentation_id': str(LS_ID),
+                        'physical_network': 'None',
                         'net_uuid': NET_UUID,
                         'network_type': 'gre'}
         self.mock_int_bridge_expected += [
@@ -521,14 +552,14 @@ class TunnelTest(object):
         self.mock_tun_bridge.add_tunnel_port.return_value = tunnel_port
         self.mock_tun_bridge_expected += [
             mock.call.add_tunnel_port('gre-0a000a01', '10.0.10.1', '10.0.0.1',
-                                      'gre', 4789, True, False),
+                                      'gre', 4789, True, False, None),
             mock.call.setup_tunnel_port('gre', tunnel_port),
         ]
 
         a = self._build_agent()
         a.tunnel_update(
             mock.sentinel.ctx, tunnel_ip='10.0.10.1',
-            tunnel_type=p_const.TYPE_GRE)
+            tunnel_type=n_const.TYPE_GRE)
         self._verify_mock_calls()
 
     def test_tunnel_update_self(self):
@@ -562,13 +593,19 @@ class TunnelTest(object):
 
         self.mock_int_bridge_expected += [
             mock.call.check_canary_table(),
+            mock.call.deferred(full_ordered=True, use_bundle=True),
+            mock.call.deferred().__enter__(),
+            mock.call.deferred().__exit__(None, None, None),
             mock.call.cleanup_flows(),
-            mock.call.check_canary_table()
-        ]
-        self.mock_tun_bridge_expected += [
-            mock.call.cleanup_flows()
+            mock.call.check_canary_table(),
+            mock.call.deferred(full_ordered=True, use_bundle=True),
+            mock.call.deferred().__enter__(),
+            mock.call.deferred().__exit__(None, None, None),
         ]
         self.mock_map_tun_bridge_expected += [
+            mock.call.cleanup_flows(),
+        ]
+        self.mock_tun_bridge_expected += [
             mock.call.cleanup_flows()
         ]
         # No cleanup is expected on ancillary bridge
@@ -634,18 +671,14 @@ class TunnelTest(object):
             self._verify_mock_calls()
 
 
-class TunnelTestOFCtl(TunnelTest, ovs_test_base.OVSOFCtlTestBase):
-    pass
-
-
-class TunnelTestRyu(TunnelTest, ovs_test_base.OVSRyuTestBase):
+class TunnelTestOSKen(TunnelTest, ovs_test_base.OVSOSKenTestBase):
     pass
 
 
 class TunnelTestUseVethInterco(TunnelTest):
     USE_VETH_INTERCONNECTION = True
 
-    def _define_expected_calls(self, arp_responder=False):
+    def _define_expected_calls(self, arp_responder=False, igmp_snooping=False):
         self.mock_int_bridge_cls_expected = [
             mock.call(self.INT_BRIDGE,
                       datapath_type=mock.ANY),
@@ -663,6 +696,7 @@ class TunnelTestUseVethInterco(TunnelTest):
             mock.call.create(),
             mock.call.set_secure_mode(),
             mock.call.setup_controllers(mock.ANY),
+            mock.call.set_igmp_snooping_state(igmp_snooping),
             mock.call.setup_default_table(),
         ]
 
@@ -694,7 +728,7 @@ class TunnelTestUseVethInterco(TunnelTest):
             mock.call.create(secure_mode=True),
             mock.call.setup_controllers(mock.ANY),
             mock.call.port_exists('patch-int'),
-            nonzero(mock.call.port_exists()),
+            mock.ANY,
             mock.call.add_patch_port('patch-int', 'patch-tun'),
         ]
         self.mock_int_bridge_expected += [
@@ -714,7 +748,7 @@ class TunnelTestUseVethInterco(TunnelTest):
         self.ipdevice_expected = [
             mock.call('int-%s' % self.MAP_TUN_BRIDGE),
             mock.call().exists(),
-            nonzero(mock.call().exists()),
+            mock.ANY,
             mock.call().link.delete()
         ]
         self.ipwrapper_expected = [
@@ -730,31 +764,36 @@ class TunnelTestUseVethInterco(TunnelTest):
         self.execute_expected = [mock.call(['udevadm', 'settle',
                                             '--timeout=10'])]
 
-
-class TunnelTestUseVethIntercoOFCtl(TunnelTestUseVethInterco,
-                                    ovs_test_base.OVSOFCtlTestBase):
-    pass
+        self.mock_int_bridge_expected += [
+            mock.call.install_goto(
+                dest_table_id=constants.LOCAL_MAC_DIRECT,
+                in_port=self.MAP_TUN_INT_OFPORT,
+                priority=4, table_id=constants.TRANSIENT_TABLE),
+            mock.call.install_goto(
+                dest_table_id=constants.LOCAL_MAC_DIRECT,
+                in_port=self.TUN_OFPORT,
+                priority=4, table_id=constants.TRANSIENT_TABLE),
+            mock.call.install_goto(
+                dest_table_id=constants.TRANSIENT_EGRESS_TABLE,
+                table_id=constants.LOCAL_MAC_DIRECT),
+        ]
 
 
-class TunnelTestUseVethIntercoRyu(TunnelTestUseVethInterco,
-                                  ovs_test_base.OVSRyuTestBase):
+class TunnelTestUseVethIntercoOSKen(TunnelTestUseVethInterco,
+                                  ovs_test_base.OVSOSKenTestBase):
     pass
 
 
 class TunnelTestWithMTU(TunnelTestUseVethInterco):
     VETH_MTU = 1500
 
-    def _define_expected_calls(self, arp_responder=False):
-        super(TunnelTestWithMTU, self)._define_expected_calls(arp_responder)
+    def _define_expected_calls(self, arp_responder=False, igmp_snooping=False):
+        super(TunnelTestWithMTU, self)._define_expected_calls(
+            arp_responder, igmp_snooping)
         self.inta_expected.append(mock.call.link.set_mtu(self.VETH_MTU))
         self.intb_expected.append(mock.call.link.set_mtu(self.VETH_MTU))
 
 
-class TunnelTestWithMTUOFCtl(TunnelTestWithMTU,
-                             ovs_test_base.OVSOFCtlTestBase):
-    pass
-
-
-class TunnelTestWithMTURyu(TunnelTestWithMTU,
-                           ovs_test_base.OVSRyuTestBase):
+class TunnelTestWithMTUOSKen(TunnelTestWithMTU,
+                           ovs_test_base.OVSOSKenTestBase):
     pass
